
local f = CreateFrame("Frame","CharDeathCounterTimer",UIParent)
local panel = CreateFrame("Frame")

local CharDeathCounterFont = "Fonts\\ARIALN.ttf"

local xpos1, ypos1, xpos2, ypos2, xpos3, ypos3 = 0
local _, englishClass, _ = UnitClass("player")
local _, _, _, argbHex = GetClassColor(englishClass)

local NAME, S = ...

SLASH_CharDeathCounter1 = "/cdc"
SLASH_CharDeathCounter2 = "/CharDeathCounter"

SlashCmdList["CharDeathCounter"] = function(msg)
	InterfaceOptionsFrame_OpenToCategory(panel);
end

function InitialiseCDCFrame()
	f:SetWidth(1) 
	f:SetHeight(1) 
	f:SetAlpha(1)
	f:RegisterForDrag("LeftButton")

	f.text = f:CreateFontString(nil,"ARTWORK") 
	f.text:SetPoint("TOPLEFT",0,0)
	f.text:SetJustifyH("LEFT")
	f.text:SetJustifyV("TOP")

	f.timer = f:CreateFontString(nil,"ARTWORK") 
	f.timer:SetPoint("TOPLEFT",f.text,"BOTTOMLEFT",0,-5)
	f.timer:SetJustifyH("LEFT")
	f.timer:SetJustifyV("TOP")

	function f:OnEvent(event, arg1, arg2)
		if (event == "ADDON_LOADED" and arg1 == NAME) then
			AddonLoadedEventHandler()
			
		elseif (event == "PLAYER_LOGIN") then
			InitialiseCharValues()
			UpdateDisplayedText()
			
		elseif (event == "PLAYER_DEAD") then
			-- fire twice if reconnecting dead ( ??? )
			if not CharDeathCounter["isDead"] then
				CharDeathCounter["deathCount"] = CharDeathCounter["deathCount"] + 1
			end
			CharDeathCounter["isDead"] = UnitIsDeadOrGhost("player")
			UpdateDisplayedText()
			
		elseif event == "PLAYER_ALIVE" or event == "PLAYER_UNGHOST" then
			CharDeathCounter["isDead"] = UnitIsDeadOrGhost("player")
		end
	end

	function f:OnUpdate(arg1)
		if CharDeathCounter["isDead"] then
			CharDeathCounter["timeSpentDead"] = CharDeathCounter["timeSpentDead"] + arg1
			f.timer:SetText(FormatedTimer())
		end
	end

	f:RegisterEvent("ADDON_LOADED")
	f:RegisterEvent("PLAYER_DEAD")
	f:RegisterEvent("PLAYER_LOGIN")
	f:RegisterEvent("PLAYER_ALIVE")
	f:RegisterEvent("PLAYER_UNGHOST")

	f:SetScript("OnEvent", f.OnEvent)
end



function UpdateDisplayedText()
	f.text:SetText(string.format("Death count: |c%s%d|r", argbHex, CharDeathCounter["deathCount"]))
	f.timer:SetText(FormatedTimer())
	f.timer:SetWidth(f.timer:GetStringWidth()+1)
end


function CharDeathCounterMovable()
	f:SetMovable(true)
	f:EnableMouse(true)
	if f.text:GetStringWidth() > f.timer:GetStringWidth() then
		f:SetWidth(f.text:GetStringWidth()+10)
	else
		f:SetWidth(f.timer:GetStringWidth()+10)
	end
	f:SetHeight(f.text:GetStringHeight()+f.timer:GetStringHeight()+5)
end

function AddonLoadedEventHandler()
	f:SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" and not self.isMoving and CharDeathCounterOptions["Move"] then
			_, _, _, xpos1, ypos1 = f:GetPoint(1)
			self:StartMoving();
			_, _, _, xpos2, ypos2 = f:GetPoint(1)
			self.isMoving = true;
		end
	end)
	f:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" and self.isMoving and CharDeathCounterOptions["Move"] then
			_, _, _, xpos3, ypos3 = f:GetPoint(1)
			self:StopMovingOrSizing()
			self.isMoving = false;
			CharDeathCounterOptions["xpos"] = xpos1 + xpos3 - xpos2
			CharDeathCounterOptions["ypos"] = ypos1 + ypos3 - ypos2
			self:SetPoint("TOPLEFT",CharDeathCounterOptions["xpos"],CharDeathCounterOptions["ypos"])
		end
	end)
	f:SetScript("OnHide", function(self)
		if self.isMoving then
			self:StopMovingOrSizing();
			self.isMoving = false;
		end
	end)

	panel.FontSlider:SetScript("OnValueChanged", function(self, newvalue)
		newvalue = floor(newvalue+0.5)
		CharDeathCounterOptions["FontSize"] = newvalue
		_G["CharDeathCounterFontSliderText"]:SetText("Font Size ("..newvalue..")");
		panel.FontSlider:SetValue(newvalue)
		f.timer:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
		f.text:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
	end)
	
	panel.ShowHideCheckButton:SetScript("OnClick",function()
		if panel.ShowHideCheckButton:GetChecked() then
			f:Show()
			CharDeathCounterOptions["Show"] = true
		else
			CharDeathCounterOptions["Show"] = false
			f:Hide()
		end
	end)
	
	panel.MoveCheckButton:SetScript("OnClick",function()
		if panel.MoveCheckButton:GetChecked() then
			CharDeathCounterOptions["Move"] = true
			CharDeathCounterMovable()
		else
			CharDeathCounterOptions["Move"] = false
			f:SetMovable(false)
			f:EnableMouse(false)
		end
	end)
	
	InitialiseOptions()
	InitialiseCharValues()
	UpdateDisplayedText()
	
	f:SetScript("OnUpdate", f.OnUpdate)
end

function InitialiseOptions()

	if CharDeathCounterOptions == nil then
		CharDeathCounterOptions = {
			xpos = 5,
			ypos = -120,
			FontSize = 13,
			Show = true,
			Move = false
		}
	end

	f:SetPoint("TOPLEFT", CharDeathCounterOptions["xpos"], CharDeathCounterOptions["ypos"])

	if CharDeathCounterOptions["Show"] then
		f:Show()
	else
		f:Hide()
	end

	f.text:SetFont(CharDeathCounterFont, CharDeathCounterOptions["FontSize"], "OUTLINE")
	f.timer:SetFont(CharDeathCounterFont, CharDeathCounterOptions["FontSize"], "OUTLINE")

	panel.FontSlider:SetValue(CharDeathCounterOptions["FontSize"])
	panel.ShowHideCheckButton:SetChecked(CharDeathCounterOptions["Show"])
	panel.MoveCheckButton:SetChecked(CharDeathCounterOptions["Move"])

	_G["CharDeathCounterFontSliderText"]:SetText("Font Size ("..CharDeathCounterOptions["FontSize"]..")");
end

function InitialiseCharValues()
	if CharDeathCounter == nil then
		CharDeathCounter = { deathCount = 0, timeSpentDead = 0 }
	end
	CharDeathCounter["isDead"] = UnitIsDeadOrGhost("player")
end

function InitialiseCDCPanel()
	panel.name = NAME
	InterfaceOptions_AddCategory(panel)

	panel.title = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	panel.title:SetPoint("TOPLEFT", 16, -16)
	panel.title:SetText(NAME)

	panel.FontSlider = CreateFrame("Slider", "CharDeathCounterFontSlider", panel, "OptionsSliderTemplate")
	panel.FontSlider:SetOrientation('HORIZONTAL')
	panel.FontSlider.tooltipText = 'Size of the text displayed on screen for CharDeathCounter'
	_G["CharDeathCounterFontSliderLow"]:SetText("10");
	_G["CharDeathCounterFontSliderHigh"]:SetText("20");
	panel.FontSlider:SetPoint("TOPLEFT", panel.title, "BOTTOMLEFT", 0, -26)
	panel.FontSlider:SetMinMaxValues(10,20)
	panel.FontSlider:SetValueStep(1)
	panel.FontSlider:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")

	panel.ShowHideCheckButton = CreateFrame("CheckButton", "CharDeathCounterShowHideCheckButton", panel, "ChatConfigCheckButtonTemplate");
	panel.ShowHideCheckButton.tooltip = "Show/Hide CharDeathCounter";
	_G["CharDeathCounterShowHideCheckButtonText"]:SetText("Show CharDeathCounter");
	panel.ShowHideCheckButton:SetPoint("TOPLEFT", panel.FontSlider, "BOTTOMLEFT", 0, -26);

	panel.MoveCheckButton = CreateFrame("CheckButton", "CharDeathCounterMoveCheckButton", panel, "ChatConfigCheckButtonTemplate");
	panel.MoveCheckButton.tooltip = "CharDeathCounter can be moved with the Interface Options window open";
	_G["CharDeathCounterMoveCheckButtonText"]:SetText("Move CharDeathCounter");
	panel.MoveCheckButton:SetPoint("TOPLEFT", panel.ShowHideCheckButton, "BOTTOMLEFT", 0, 0);

	panel.default = function (self)
		InitialiseOptions()
		UpdateDisplayedText()
		print("[CharDeathCounter] Settings have been reset to default.")
	end
end


function FormatedTimer()
	local seconds = math.floor(CharDeathCounter["timeSpentDead"])
	local hours = "00"
	local mins = "00"
	local secs = "00"
	
	if seconds > 0 then
		hours = string.format("%02.f", math.floor(seconds / 3600))
		mins = string.format("%02.f", math.floor(seconds / 60 - ( hours * 60)))
		secs = string.format("%02.f", math.floor(seconds % 60))
	end
	
	return string.format("Time spent dead: |c%s%s:%s:%s|r", argbHex, hours, mins, secs)
end


InitialiseCDCFrame()
InitialiseCDCPanel()
